
#include "configuration.h"
#include "pins.h"
#include "intercom.h"
#include "extruder.h" 



// Select a new extruder
void newExtruder(byte e)
{ 
  if(e < 0)    return;
  if(e >= EXTRUDER_COUNT)    return;
  if(e != extruder_in_use)  {      
    extruder_in_use = e;    
    setUnits(cdda[0]->get_units());  
  }
}


/*************************************************************************
 * Extruder functions that are the same for all extruders.
 **************************************************************************/
// The number of 5-second intervals to wait at the target temperature for things to stabilise.
// Too short, and the extruder will jam as only part of it will be hot enough.
// Too long and the melt will extend too far up the insulating tube.
// Default value: 10
#define WAIT_AT_TEMPERATURE 10

// The temperature control dead zone in C deg. ORIGINAL was 5
#define HALF_DEAD_ZONE 20

void extruder::waitForTemperature()
{ 
  byte seconds = 0;  
  bool warming = true;  
  count = 0;  
  newT = 0;  
  oldT = newT;
  while (true) {
    newT += getTemperature();  
    count++; 
    if(count > 5)  {
      newT = newT/5; 
      if(newT >= targetTemperature - HALF_DEAD_ZONE)  { 
        warming = false;   
        if(seconds > WAIT_AT_TEMPERATURE) return; 
        else seconds++; 
      } 
      if(warming) { 
        if(newT > oldT) oldT = newT; 
        else { /* Temp isn't increasing - extruder hardware error */
          temperatureError();  
          return; 
        } 
      }
      newT = 0; 
      count = 0;
    }    
    for(int i = 0; i < 1000; i++)    
    {      
      manage();      
      delay(1);    
    } 
  }
}

// This is a fatal error - something is wrong with the heater.
void extruder::temperatureError()
{  
  sprintf(talkToHost.string(), "Extruder temperature not rising - hard fault.");  
  talkToHost.setFatal(); 
}

//***************************************************************************************************************************


